import {
    Args,
    Context,
    Mutation,
    Parent,
    Query,
    ResolveField,
    Resolver,
    Subscription,
} from '@nestjs/graphql';
import { Group } from './models/group.model';
import { PrismaService } from '../prisma/prisma.service';
import { GraphQLString } from 'graphql';
import * as seedWords from 'mnemonic-words';
import { ForbiddenException, NotFoundException, UnauthorizedException, UseGuards } from '@nestjs/common';
import { GroupAuthGuard } from '../auth/group-auth.guard';
import { Request } from 'express';
import { pubSub } from '../pubSub.instance';
import { AgentAuthGuard } from 'src/auth/agent-auth.guard';

@Resolver(() => Group)
export class GroupResolver {
    constructor(private prismaService: PrismaService) {}

    @Subscription(() => Group, {
        filter: (payload, variables, state) =>
            payload.updateToken.code === variables.groupCode,
    })
    async updateToken(
        @Args({ name: 'groupCode', type: () => GraphQLString })
        groupCode: string,
    ) {
        if (!groupCode) throw new UnauthorizedException();

        const group = await this.prismaService.group.findFirst({
            where: {
                code: groupCode,
            },
        });

        if (!group) throw new ForbiddenException();

        return pubSub.asyncIterator('updateToken');
    }

    @Mutation(() => Group)
    async addToken(
        @Args({ name: 'code', type: () => GraphQLString }) code: string,
    ) {
        const group = await this.prismaService.group.update({
            where: {
                code: code,
            },
            data: {
                tokens: { increment: 1 },
            },
        });

        if (!group) throw new NotFoundException();

        await pubSub.publish('updateToken', { updateToken: group });

        return group;
    }

    @Mutation(() => Group)
    async createGroup(
        @Args({ name: 'name', type: () => GraphQLString }) name: string,
        @Args({ name: 'code', type: () => GraphQLString }) agentCode: string,
    ) {
        if (!agentCode) throw new UnauthorizedException();

        // TODO: refactor agent auth, modify auth guard

        const agent = await this.prismaService.agent.findFirst({
            where: {
                slug: agentCode,
            },
        });

        if (!agent) throw new ForbiddenException();

        function generateWord() {
            return seedWords[Math.round(Math.random() * seedWords.length)];
        }

        const code = [
            generateWord(),
            generateWord(),
            generateWord(),
            generateWord(),
        ].join('-');

        return this.prismaService.group.create({
            data: {
                name,
                code,
            },
        });
    }

    @Query(() => Group)
    @UseGuards(GroupAuthGuard)
    async getGroup(@Context() { req }: { req: Request }) {
        return req.group;
    }

    @ResolveField('unlocks')
    async unlocks(@Parent() group: Group) {
        const result = await this.prismaService.group.findFirst({
            where: { id: group.id },
            select: { unlocks: true },
        });

        return result.unlocks.map((unlock) => ({
            ...unlock,
            locked: false,
        }));
    }
}
